/******************************************

    "matrix.cpp" - Simple matrix manipulation using Class
    Copyright (C) 2014  Niran Meher

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>

******************************************/

# include <iostream>
# include <cstdio>

using namespace std ;

class Exception
{
   	public:
      		const char* msg;
      		Exception(const char* arg) : msg(arg)
      		{ 
		}
};

class Matrix
{
	private :
	
		int rows ;
		int cols ;
		double ** p ;

	public :
		
		Matrix ()
    		{
        		p = NULL ;
        		rows = 0 ;
        		cols = 0 ;
    		}

		Matrix (const int row_count, const int column_count)
    		{
        		p = NULL;

        		if (row_count > 0 && column_count > 0)
        		{
            			rows = row_count;
            			cols = column_count;

            			p = new double*[rows];
            			for (int r = 0; r < rows; r++)
            			{
                			p[r] = new double[cols];
                			for (int c = 0; c < cols; c++)
                    				p[r][c] = 0;
                			
            			}
        		}
    		}

		Matrix (const Matrix& a)
    		{
        		rows = a.rows ;
        		cols = a.cols ;
        		p = new double*[a.rows];
        		for (int r = 0; r < a.rows; r++)
        		{
            			p[r] = new double[a.cols];

            			for (int c = 0; c < a.cols; c++)
            			{
                			p[r][c] = a.p[r][c];
            			}
        		}
    		}

		int get_cols ()
		{
			return this->cols ;
		}

		int get_rows ()
		{
			return this->rows ;
		}

		double get_value (int r, int c)
		{
			if (p != NULL && r >= 0 && r < rows && c >= 0 && c < cols)
			{
				return p[r][c] ;
			}
			else
			{
				cerr << "Subscript out of range" ;
				return -1 ;
			}
		}

		double & operator () (const int r, const int c)
		{
			return this->p[r][c] ;
		}

		friend istream& operator >> (istream& input, const Matrix& a)
		{
			for (int i = 0; i < a.rows; i++)
				for (int j = 0; j < a.cols; j++)
					input >> a.p[i][j] ;
		}

		friend ostream& operator << (ostream& output, const Matrix& a)
		{
			for (int i = 0; i < a.rows; i++)
			{
				for (int j = 0; j < a.cols; j++)
					output << a.p[i][j] << " " ;
				cout << endl ;
			}
		}
		
		friend Matrix operator * (const Matrix& a, const Matrix& b)
		{
			if (a.cols == b.rows)
			{
				Matrix result = Matrix (a.rows, b.cols) ;

				for (int r = 0; r < a.rows; r++)
            			{
                			for (int c_res = 0; c_res < b.cols; c_res++)
                			{
                    				for (int c = 0; c < a.cols; c++)
                    				{
                        				result.p[r][c_res] += a.p[r][c] * b.p[c][c_res];
                    				}
                			}
            			}
				return result ;
			}
			else
			{
				cerr << "\nError " ;
				return Matrix(0,0);
			}
		}

		friend Matrix operator + (const Matrix& a, const Matrix& b)
		{
			if (a.cols == b.cols && a.rows == b.rows)
			{
				Matrix result = Matrix (a.rows, a.cols) ;

				for (int i = 0; i < a.rows; i++)
					for (int j = 0; j < a.cols; j++)
						result.p[i][j] = a.p[i][j] + b.p[i][j] ;
				return result ;
			}
			else
			{
				cerr << "\nDimension illegal\n" ;
				return Matrix(0,0) ;
			}
		}

		friend Matrix operator - (const Matrix& a, const Matrix& b)
		{
			if (a.cols == b.cols && a.rows == b.rows)
			{
				Matrix result = Matrix (a.rows, a.cols) ;

				for (int i = 0; i < a.rows; i++)
					for (int j = 0; j < a.cols; j++)
						result.p[i][j] = a.p[i][j] - b.p[i][j] ;
				return result ;
			}
			else
			{
				cerr << "\nDimension illegal\n" ;
				return Matrix(0,0) ;
			}
		}
/**
		Matrix& Multiply (const double a)
		{
			for (int i = 0; i < a.rows; i++)
				for (int j = 0; j < a.cols; j++)
					a.p[i][j] *= a ;
			return *this ;
		}

		Matrix& Add (const double a)
		{
			for (int i = 0; i < a.rows; i++)
				for (int j = 0; j < a.cols; j++)
					a.p[i][j] += a ;
			return *this ;
		}
**/
		Matrix Minor(const int row, const int col) const
    		{
        		Matrix res;
            		res = Matrix(rows - 1, cols - 1);

            		for (int r = 0; r < (rows - (row >= rows)); r++)
            		{
                		for (int c = 0; c < (cols - (col >= cols)); c++)
                		{
                    			res(r - (r > row), c - (c > col)) = p[r][c];
                		}
            		}
        		
        		return res;
    		}

		friend double Det ( Matrix &a) ;
/**
		Matrix& Multiply (const double a)
		{
			return Add (-a) ;
		}			
		
		Matrix& Divide (const double a)
		{
			if ( a != 0 )
				return Multiply (1/a) ;
			else
			{
				cerr << "\nDividing by Zero\n" ;
				return Matrix(0,0) ;
			}
		}**/
};


void add_subtract_matrix ()
{
	int a_row, a_column, b_row, b_column ;
	char operation ; 
	
	cout << "\nEnter rows and columns for matrix A and B respectively :\n" ;
	cin >> a_row >> a_column >> b_row >> b_column ;

	Matrix A(a_row, a_column), B(b_row, b_column) ;

	if (a_row != b_row || a_column != b_column)
	{
		cout << "\nError : Dimensions do not match" ;
		return ;
	}

	cout << "\nEnter matrix A :\n" ;
	cin >> A ;

	cout << "\nEnter matrix B :\n" ;
	cin >> B ;

	cout << "\nFor addition or subtraction press '+' or '-' : " ;
	cin >> operation ;

	if (operation == '+')
	{
		Matrix C(a_row,b_column) ;
		C = A + B ;

		cout << C ;
	}
	else if (operation == '-')
	{
		Matrix C(a_row,b_column) ;
		C = A - B ;

		cout <<"Addition is :\n";
		cout << C ;
	}
}

void multiply_matrix ()
{
	int a_row, a_column, b_row, b_column ;
	
	cout << "\nEnter rows and columns for matrix A and B respectively :\n" ;
	cin >> a_row >> a_column >> b_row >> b_column ;

	Matrix A(a_row, a_column), B(b_row, b_column) ;

	if (a_column != b_row)
	{
		cout << "\nError : Dimensions do not match" ;
		return ;
	}

	cout << "\nEnter matrix A :\n" ;
	cin >> A ;

	cout << "\nEnter matrix B :\n" ;
	cin >> B ;

	Matrix C(a_row, b_column) ;
	
	C = A * B ;

	cout << "Multiplication is :\n" ;	
	cout << C ;
}

double Det( Matrix& a)
{
	double d = 0 ;       // value of the determinant
    	int rows = a.get_rows () ;
    	int cols = a.get_cols () ;
        	
        if (rows == 1)
        {
            d = a.get_value(0, 0);
        }
        else if (rows == 2)
        {
            d = a.get_value(0, 0) * a.get_value(1, 1) - a.get_value(1, 0) * a.get_value(0, 1);
        }
        else
        {
            	for (int c = 0; c < cols; c++)
            	{
                	Matrix M = a.Minor(0, c);
                	d += (c%2 + c%2 - 1) * a.get_value(0, c) * Det(M);
            	}
        }
	return d ;
}

void determinant ()
{
	int  n ; 

	cout << "\nEnter dimension :\n" ;
	cin >> n ;
	
	Matrix A(n,n) ;

	cout << "\nEnter matrix :\n" ;

	cin >> A ;

	cout << "\nDeterminant is : " << Det (A)*(-1) ;

	return ;
}

void matrix_operation ()
{
	cout << "Menu driven Matrix manipulation program :\n" ;

	while (true)
	{
		int c ;

		cout << "\n 1. Add or Subtract Matrix\n 2. Multiply two Matrices\n 3. Find determinant (upto 3x3)\n 4. Exit \n:" ;
		cin >> c ;
		
		switch (c)
		{
			case 1: add_subtract_matrix () ;
				break ;
			case 2: multiply_matrix () ;
				break ;
			case 3: determinant () ;
				break ;
			case 4: //inverse () ;
				return ;
			default: cout << "\nInvalid option\n" ;
		}
	}
}

int main (void)
{

	matrix_operation () ;

	return 0 ;
}




